SystemOrganization addCategory: #'Phidget-Core'!

Object subclass: #PhdCrPhidget
	instanceVariableNames: 'semaphores processBlocks opened handle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrPhidget commentStamp: 'pt 5/3/2009 14:26' prior: 0!
A PhdCrPhidget is an abstract class that provides some primitives and methods for Phidget support. Please read the comments of my subclasses to find more information how to use Phidgets in Squeak.

Instance Variables
	opened:			<Boolean>
	processBlocks:	<Dictionary>
	semaphores:		<Dictionary>

opened
	- true, if the Phidget was opened - Squeak is able to communicate with the hardware.
	- false, if the Phidgets was not opened or was opened and then closed.

semaphores
	- A Dictionary with entries of the following form: #aSymbol -> aSemaphore.
	#aSymbol may be #Attach, #Detach, #Error for Phidgets in general. Other symbols may excist for phidget specific support.
	This dictionary is used to handle the callbacks from the Phidget C API.
	These semaphores can be registered for certain events (like the attach event) - so that the C API signals is if the event occours. These semaphores are used in the callbackProcess method of this class.

processBlocks
	- A Dictionary with entries of the following form: #aSymbol -> aBlock.
	The symbols are equivalent to the symbols used in the semaphores dictionary.
	This dictionary is used to handle the callbacks from the Phidget C API.
	Each time a semaphore from the semaphores dictionary was signaled from the C API, the corresponding block vom this dictionary will be evaluated by the callbackProcess method of this class.
	!

PhdCrPhidget subclass: #PhdCrInterfaceKit
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrInterfaceKit commentStamp: 'pt 5/3/2009 15:04' prior: 0!
A PhdCrInterfaceKit is a representation of a Phidget called 'Interface Kit' in Squeak.
To use the Interface Kit, create a new instance of this class and open a connection to the hardware board with:
	ifKit := PhdCrInterfaceKit new.
	ifKit openAny.
A block can be set to be evaluated every time the hardware board was attached, detached, an error occoured, a sensor value was changed, an input value was changed or an output value was changed. For example:
	ifKit
		onAttachDo: [ Transcript show:'servo was attached!!;cr' ];
		onDetachDo: [ Transcript show:'servo was detached!!;cr' ];
		onSensorChangeDo: [ Transcript show:'A sensor changed it's value!!' ].

Depending on which hardware is attached it is possible to attach one or more analog sensors (, digital inputs or digital outputs) to the board. To get the number of available sensors send 'sensorCount' ('inputCount', 'outputCount'). Every sensor can be accessed by an index adressing it. Valid indices are greater or equal than 1 and less or equal than the number of available sensors (sensorCount).
To get the value of a sensor, call 'sensorValue: index'. The value will be a number between 0 and 1000.
The input and output values are boolean and may be read by: 'outputValue: index' or 'inputValue: index'.
It is possible to set an output value to true/false with 'setOutputValue: aBoolean at: index'.

IMPORTANT:
After using the Interface Kit it is important to close the connection to the hardware by sending 'close':
	ifKit close.

NOTE:
The sensor changeBlock will be evaluated only, it the change is greater than a number called sensitivity. Example: A light sensor probably changes its value continuously. We want to know greater changes only, so we set the sensitivity to 50. So the sensor change block will be evaluated only if the change is greater than 50. To set the sensitivity, call 'sensitivity: aNumber to: index'.


Instance Variables
	ifKit:		<ByteArray>

ifKit
	- a ByteArray that contains the Interface Kit handle for the C API. Every primitive (except the one where the handle is created) needs this ByteArray to let the C API know about which Interface Kit it is currently talking.
!

!PhdCrInterfaceKit class methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:23'!
knownCallbacks
	"Returns a list of callback names (#symbols) the InterfaceKit knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #SensorChange #InputChange #OutputChange )! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:17'!
inputCount
	"Returns the number of inputs that may be attached to the Interface Kit."
	
	self attached
		ifTrue: [ ^ self primNumInputs: handle ].
	^ 0! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:28'!
inputIndexIsInRange: index
	"Returns true if 1<=index<=sensorCount.
	Every time a primitive is used that needs the input index, this method should be used to check if the index is in the correct range."
	
	index isInteger ifFalse: [ ^ false ].
	^ ( 1 <= index ) and: [ index <= self inputCount ]! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:18'!
inputValue: inputIndex
	"Gets the output value (true or false) at the digital output with the given index."

	( self inputIndexIsInRange: inputIndex ) ifFalse: [ ^ nil ].
	( ( self primInputValue: handle at: inputIndex - 1 ) == 1 ) ifTrue: [ ^ true ].
	^ false! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:58'!
onInputChangeDo: aBlock
	"The aBlock will be evaluated if an input changed it's state."

	processBlocks at: #InputChange put: aBlock.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:59'!
onOutputChangeDo: aBlock
	"The aBlock will be evaluated if an output changed it's state."

	processBlocks at: #OutputChange put: aBlock.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:59'!
onSensorChangeDo: aBlock
	"The aBlock will be evaluated if a sensor changed it's value."

	processBlocks at: #SensorChange put: aBlock.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:19'!
outputCount
	"Returns the number of outputs that may be attached to the Interface Kit."
	
	self attached
		ifTrue: [ ^ self primNumOutputs: handle ].
	^ 0! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:27'!
outputIndexIsInRange: index
	"Returns true if 1<=index<=sensorCount.
	Every time a primitive is used that needs the output index, this method should be used to check if the index is in the correct range."
	
	index isInteger ifFalse: [ ^ false ].
	^ ( 1 <= index ) and: [ index <= self outputCount ]! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:19'!
outputValue: outputIndex
	"Gets the output value (true or false) at the digital output with the given index."

	( self outputIndexIsInRange: outputIndex ) ifFalse: [ ^ nil ].
	( ( self primOutputValue: handle at: outputIndex - 1 ) == 1 ) ifTrue: [ ^ true ].
	^ false! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:01'!
primAttached: phid
	"Return 1 if the Interface Kit is attached - otherweise return 0."
	
	<primitive: 'primIfKitAttached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/12/2009 12:28'!
primClose: phid
	"Close the Interface Kit."
	
	<primitive: 'primIfKitClose' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:02'!
primCreate
	"Reset the Interface Kit (Semaphores and Handle)."

	<primitive: 'primIfKitCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/3/2009 00:05'!
primGetSensorChangeTrigger: phid at: sensorIndex
	"Gets the sensor change trigger. A sensor change will be registered if the change is higher than this number only."

	<primitive: 'primIfKitGetSensorChangeTrigger' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:37'!
primInputValue: phid at: sensorIndex
	"Gets the input value (0, 1) at the input with the given index."

	<primitive: 'primIfKitInputValue' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/6/2009 17:06'!
primNumInputs: phid
	"Gets the number of inputs."

	<primitive: 'primIfKitNumInput' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/6/2009 17:06'!
primNumOutputs: phid
	"Gets the number of outputs."

	<primitive: 'primIfKitNumOutput' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:06'!
primNumSensors: phid
	"Gets the number of sensors."

	<primitive: 'primIfKitNumSensors' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:14'!
primOnAttached: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when the Interface Kit becomes attached."
	
	<primitive: 'primIfKitOnAttached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:14'!
primOnDetached: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when the Interface Kit becomes detached."
	
	<primitive: 'primIfKitOnDetached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:14'!
primOnError: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when a an Error occours. Use primLastError to get the last error code"
	
	<primitive: 'primIfKitOnError' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:14'!
primOnInputChanged: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when an InputValue changed."
	
	<primitive: 'primIfKitOnInputChanged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:15'!
primOnOutputChanged: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when an OutputValue changed."
	
	<primitive: 'primIfKitOnOutputChanged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:15'!
primOnSensorChanged: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when a SensorValue changed."
	
	<primitive: 'primIfKitOnSensorChanged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:11'!
primOpen: phid withSerial: serial
	"Opens the Interface Kit with the given serial number"

	<primitive: 'primIfKitOpen' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:11'!
primOpenAny: phid
	"Opens any available Interface Kit"

	<primitive: 'primIfKitOpenAny' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:37'!
primOutputValue: phid at: sensorIndex
	"Gets the output value (0, 1) at the output with the given index."

	<primitive: 'primIfKitOutputValue' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:12'!
primSensorValue: phid at: sensorIndex
	"Gets the sensor value (0-1000) at the analog input with the given index."

	<primitive: 'primIfKitSensorValue' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/6/2009 17:12'!
primSetOutput: phid at: sensorIndex to: aNumber
	"Sets the output to  0/1 (which means ON/OFF) at the output with the given index."

	<primitive: 'primIfKitSetOutput' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/3/2009 00:02'!
primSetSensorChangeTrigger: phid at: sensorIndex to: aNumber
	"Sets the sensor change trigger to  aNumber (0-1000). A sensor change will be registered if the change is higher than aNumber only."

	<primitive: 'primIfKitSetSensorChangeTrigger' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/13/2009 11:53'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding semaphores for #SensorChange, #InputChange and #OutputChange."

	| semaIndex |
	super registerSemaphores.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#SensorChange ).
	self primOnSensorChanged: handle semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#InputChange ).
	self primOnInputChanged: handle semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#OutputChange ).
	self primOnOutputChanged: handle semaphoreIndex: semaIndex.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:20'!
sensitivity: sensitivity at: index 
	"Sets the input sensitivity. The onSensorChange block will be evaluated only, if the change of the sensor value is greater than the input sensitivity."
	(self sensorIndexIsInRange: index) ifFalse: [ ^ false ].
	^ self 
		primSetSensorChangeTrigger: handle
		at: index - 1
		to: sensitivity! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:20'!
sensitivityAt: index 
	"Returns the input sensitivity. The onSensorChange block will be evaluated only, if the change of the sensor value is greater than the input sensitivity."
	(self sensorIndexIsInRange: index) ifFalse: [ ^ false ].
	^ self 
		primGetSensorChangeTrigger: handle
		at: index - 1! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:20'!
sensorCount
	"Returns the number of sensors that may be attached to the Interface Kit."
	self attached
		ifTrue: [ ^ self primNumSensors: handle ].
	 ^ 0! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:27'!
sensorIndexIsInRange: index
	"Returns true if 1<=index<=sensorCount.
	Every time a primitive is used that needs the sensor index, this method should be used to check if the index is in the correct range."
	
	index isInteger ifFalse: [ ^ false ].
	^ ( 1 <= index ) and: [ index <= self sensorCount ]! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:20'!
sensorValue: sensorIndex
	"Gets the sensor value (0-1000) at the analog input with the given index."

	(self sensorIndexIsInRange: sensorIndex) ifFalse: [ ^ nil ].
	^ self primSensorValue: handle at: sensorIndex - 1! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:20'!
setOutputValue: aBoolean at: outputIndex
	"Sets the output value (true or false) at the digital output with the given index."

	| anInteger |
	( self outputIndexIsInRange: outputIndex ) ifFalse: [ ^ nil ].
	aBoolean
		ifTrue: [ anInteger := 1 ]
		ifFalse: [ anInteger := 0 ].
	^ self primSetOutput: handle at: outputIndex-1 to: anInteger.! !

!PhdCrPhidget class methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:23'!
knownCallbacks
	"Returns a list of callback names (#symbols) the Phidget knows"
	
	^ #( #Attach #Detach #Error )! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 11:29'!
attached
	"Returns true if the Phidget is attached, and false otherwise."
	
	( ( self primAttached: handle ) == 1 ) ifTrue: [ ^ true ].
	^ false! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:51'!
callbackProcess: aSymbol
	"This method will be forked as an extra process for every callback that a phidget needs.
	aSymbol can be #Attach, #Detach, #Error for every phidget. Other phidget-dependent symbols have to be set (like in this initialise method) in the phidgets initialise method"

	( semaphores at: aSymbol ) wait.
	[opened] whileTrue: [
		( processBlocks at: aSymbol ) value.
		( semaphores at: aSymbol ) wait ].! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 11:33'!
close
	"Close the Phidget."

	opened ifFalse: [^ self]. "already closed"
	self primClose: handle.
	opened := false.
	self unregisterSemaphores.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 11:58'!
initialize
	"Initialise the callbacks that are used in every Phidget.
	Every subclass should call super initialise or initialise the callbacks by their own."

	super initialize.
	
	semaphores := Dictionary new.
	processBlocks := Dictionary new.
	
	self class knownCallbacks do: [ :name |
		semaphores add: name -> Semaphore new.
		processBlocks add: name -> [] ].
		
	opened := false.
	handle := self primCreate.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:10'!
lastError
	"Returns the error code of the last error.
	Descriptions of the codes may be found on www.phidgets.com or phidget21.h on line 2250 ff."

	^ self primLastError! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:43'!
onAttachDo: aBlock
	"The aBlock will be evaluated if the phidget was attached."

	processBlocks at: #Attach put: aBlock.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:43'!
onDetachDo: aBlock
	"The aBlock will be evaluated if the phidget was detached."

	processBlocks at: #Detach put: aBlock.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:45'!
onErrorDo: aBlock
	"The aBlock will be evaluated if an error occoured with the phidget."

	processBlocks at: #Error put: aBlock.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:26'!
openAny
	"Opens any Phidget."
	
	handle ifNil: [ self error: 'Could not open Phidget!!'. ^ self ].
	
	self
		registerSemaphores;
		primOpenAny: handle.
	opened := true.
	self class knownCallbacks do: [ :name |
		[ self callbackProcess: name ] fork ].! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:26'!
openSpecefic: serialNumber
	"Opens the Phidget with the given serial number."
	
	handle ifNil: [ self error: 'Could not open Phidget!!'. ^ self ].
	
	self
		registerSemaphores;
		primOpen: handle
		withSerial: serialNumber.
	opened := true.
	self class knownCallbacks do: [ :name |
		[ self callbackProcess: name ] fork ].! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 11:31'!
primAttached: aPhidgetHandle
	"Returns true if the Phidget is attached, and false otherwise."
	
	self subclassResponsibility.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 11:32'!
primClose: aPhidgetHandle
	"Close the connection to the Phidget."
	
	self subclassResponsibility.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 11:57'!
primCreate
	"Reset the Phidget (Semaphores and Handle)."
	
	self subclassResponsibility.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:10'!
primLastError
	"Returns the error code of the last error.
	Descriptions of the codes may be found on www.phidgets.com or phidget21.h on line 2250 ff."

	<primitive: 'primLastError' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:07'!
primOpen: phid withSerial: serial
	"Opens the Phidget with the given serial number"

	self subclassResponsibility.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:06'!
primOpenAny: phid
	"Opens any available Phidget"

	self subclassResponsibility.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'lk 5/13/2009 11:51'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. May be extended in subclasses."

	| semaIndex |
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#Attach ).
	self primOnAttached: handle semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#Detach ).
	self primOnDetached: handle semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#Error ).
	self primOnError: handle semaphoreIndex: semaIndex.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:16'!
unregisterSemaphores
	"Unregister the semaphores which were previously registered with 'self registerSemaphores'."

	semaphores do: [ :sema |
		sema signal.
		Smalltalk unregisterExternalObject: sema ]! !

PhdCrPhidget subclass: #PhdCrServo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrServo commentStamp: 'pt 5/3/2009 15:04' prior: 0!
A PhdCrServo is a representation of a Phidget called 'Servo' (here: 'Servo board') in Squeak.
To use the Servo board, create a new instance of this class and open a connection to the hardware board with:
	servo := PhdCrServo new.
	servo openAny.
A block can be set to be evaluated every time the hardware board was attached, detached, an error occoured or the position of a motor was changed. For example:
	servo
		onAttachDo: [ Transcript show:'servo was attached!!;cr' ];
		onDetachDo: [ Transcript show:'servo was detached!!;cr' ];
		onPositionChangeDo: [ Transcript show:'A motor changed it's position!!' ].
Depending on which hardware is attached it is possible to attach one or more servo motors to the board. To get the number of available motors send 'motorCount'. Every motor can be accessed by an index adressing the motor. Valid indices are greater or equal than 1 and less or equal than the number of available motors (motorCount).
To set the position of a motor to a value, call 'setMotorPosition: aFloat at: index' (aFloat must be between minMotorPositionAt: index and maxMotorPositionAt: index !!).

IMPORTANT:
After using the servo it is important to close the connection to the hardware by sending 'close':
	servo close.

Instance Variables
	servo:		<ByteArray>

servo
	- a ByteArray that contains the Servo board handle for the C API. Every primitive (except the one where the handle is created) needs this ByteArray to let the C API know about which Servo board it is currently talking.
!

!PhdCrServo class methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:23'!
knownCallbacks
	"Returns a list of callback names (#symbols) the Servo knows.
	This extends the list the super class knows."
	
	^ super knownCallbacks, #( #PositionChange )! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:20'!
getEngagedAt: index 
	"Returns the engaged state of a motor. This is whether the motor is powered or not."
	
	(self indexIsInRange: index) ifFalse: [ ^ self ].
	(self 
		primGetEngaged: handle
		at: index - 1) == 1 ifTrue: [ ^ true ].
	^ false! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:26'!
indexIsInRange: index 
	"Returns true if 1<=index<=motorCount.
	Every time a primitive is used that needs the motor index, this method should be used to check if the index is in the correct range."
	index isInteger ifFalse: [ ^ false ].
	^ 1 <= index and: [ index <= self motorCount ]! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'lk 5/14/2009 21:36'!
maxMotorPositionAt: index 
	"Returns the max. allowed motor position for the motor attached at the given index."
	
	(self indexIsInRange: index) ifFalse: [ ^ 0 ].
	^ self 
		primGetPositionMax: handle
		at: index - 1! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'lk 5/14/2009 21:36'!
minMotorPositionAt: index 
	"Returns the min. allowed motor position for the motor attached at the given index."
	(self indexIsInRange: index) ifFalse: [ ^ 0 ].
	^ self 
		primGetPositionMin: handle
		at: index - 1! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:21'!
motorCount
	"Returns the number of motors that may be attached to the Servo board."
	
	self attached
		ifTrue: [ ^ self primNumMotors: handle ].
	^ 0! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'lk 5/14/2009 21:36'!
motorPositionAt: index 
	"Returns the current position of the motor attached at the given index."
	(self indexIsInRange: index) ifFalse: [ ^ 0 ].
	^ self 
		primGetPosition: handle
		at: index - 1! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:43'!
onPositionChangeDo: aBlock
	"The aBlock will be evaluated if the position of a motor was changed."

	processBlocks at: #PositionChange put: aBlock.! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/4/2009 15:31'!
positionIsInRange: position at: index 
	"The position is in range, if the index is in range and the position is between the min. and max. Position."
	(self indexIsInRange: index) ifFalse: [ ^ false ].
	(self maxMotorPositionAt: index) >= position ifFalse: [ ^ false ].
	(self minMotorPositionAt: index) <= position ifFalse: [ ^ false ].
	^ true! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:24'!
primAttached: phid
	"Return 1 if the Servo board ist attached - otherwise return 0."

	<primitive: 'primServoAttached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:24'!
primClose: phid
	"Close the connection to the Servo board."

	<primitive: 'primServoClose' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:24'!
primCreate
	"Reset the Servo board (Semaphores and Handle)."

	<primitive: 'primServoCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 22:21'!
primGetEngaged: phid at: index
	"Returns 1 if the motor on the given index is engaged - otherwise return 0.
	This is whether the motor is powered or not."

	<primitive: 'primServoGetEngaged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/14/2009 21:35'!
primGetPosition: phid at: index
	"Returns a float containing the position of the motor at the given index."

	<primitive: 'primServoGetPosition' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/14/2009 21:34'!
primGetPositionMax: phid at: index
	"Returns a float containing max. possible position of the motor at the given index."

	<primitive: 'primServoGetPositionMax' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/14/2009 21:35'!
primGetPositionMin: phid at: index
	"Returns a float containing min. possible position of the motor at the given index."

	<primitive: 'primServoGetPositionMin' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:24'!
primNumMotors: phid
	"Returns the number of motors that can be attached to the Servo board."

	<primitive: 'primServoNumMotors' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:16'!
primOnAttached: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when the servo board becomes attached."
	
	<primitive: 'primServoOnAttached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:16'!
primOnDetached: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when the Servo board becomes detached."
	
	<primitive: 'primServoOnDetached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:16'!
primOnError: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when a an Error occours. Use primLastError to get the last error code"
	
	<primitive: 'primServoOnError' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:16'!
primOnPositionChanged: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when a motor changes its position."
	
	<primitive: 'primServoOnPositionChanged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:26'!
primOpen: phid withSerial: serial
	"Opens the Servo board with the given serialNumber."

	<primitive: 'primServoOpen' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:26'!
primOpenAny: phid
	"Opens any available Servo board."

	<primitive: 'primServoOpenAny' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 4/28/2009 10:13'!
primSetEngaged: phid at: index to: value
	"Sets the engaged state of the motor at the given index to <value>. Value must be 0 or 1!!
	This is whether the motor is powered or not."

	<primitive: 'primServoSetEngaged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/14/2009 21:38'!
primSetPosition: phid at: index to: aFloat
	"Set the position of the motor at the given index to aFloat. aFloat must be between primGetPositionMin and primGetPositionMax."

	<primitive: 'primServoSetPosition' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'lk 5/13/2009 11:54'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them. Extends the superclasses' method by adding a semaphore for #PositionChange."

	| semaIndex |
	super registerSemaphores.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#PositionChange ).
	self primOnPositionChanged: handle semaphoreIndex: semaIndex.! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'lk 5/12/2009 12:21'!
setEngaged: aBoolean at: index 
	"Sets the engaged state of a motor. This is whether the motor is powered or not."
	| anInt |
	(self indexIsInRange: index) ifFalse: [ ^ self ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	self 
		primSetEngaged: handle
		at: index - 1
		to: anInt! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'lk 5/14/2009 21:39'!
setPosition: aValue at: index
	"Set the position of the motor attached at <index> to <aValue>.
	aValue should be a float betwenn the min. and max. motor position."

	| aFloat |
	aFloat := aValue asFloat.
	( self positionIsInRange: aFloat at: index ) ifFalse: [ ^ self ].
	^ self primSetPosition: handle at: index - 1 to: aFloat! !
